// Copyright (c) 2023 Dominic Masters
// 
// This software is released under the MIT License.
// https://opensource.org/licenses/MIT

#include "PlayerController.hpp"
#include "game/DawnGame.hpp"

using namespace Dawn;

PlayerController::PlayerController(SceneItem *item) : SceneItemComponent(item) {

}

void PlayerController::onStart() {
  useEvent([&](float_t delta){
    auto camera = getScene()->findComponent<Camera>();
    assertNotNull(camera);

    // Friction
    velocity -= velocity * friction * delta;

    // Movement
    auto inputMove = getGame()->inputManager.getAxis2D(
      INPUT_BIND_NEGATIVE_X, INPUT_BIND_POSITIVE_X,
      INPUT_BIND_NEGATIVE_Y, INPUT_BIND_POSITIVE_Y
    ) * delta * moveSpeed;
    
    if(inputMove.x != 0 || inputMove.y != 0) {
      float_t angle = atan2(inputMove.y, inputMove.x);
      glm::vec3 movement = glm::vec3(cos(angle), 0, sin(angle));
      velocity += movement * delta * moveSpeed;
    }

    // Move / Update
    transform->setLocalPosition(transform->getLocalPosition() + (velocity * delta));
  }, getScene()->eventSceneUpdate);
}